/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ra;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;
import java.util.logging.Logger;

import org.drools.FactException;
import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.WorkingMemory;
import org.drools.compiler.DroolsParserException;
import org.drools.compiler.PackageBuilder;
import org.drools.compiler.PackageBuilderConfiguration;

public class DroolsWorkingMemory implements Runnable {

    private static Logger logger = Logger.getLogger(DroolsWorkingMemory.class
            .getName());

    private static DroolsWorkingMemory instance = null;

    private WorkingMemory workingMemory = null;

    private long timeout = 5000;

    private boolean started = true;

    public static DroolsWorkingMemory getInstance() {
        if (instance == null) {
            instance = new DroolsWorkingMemory();
        }
        return instance;
    }

    private DroolsWorkingMemory() {
        new Thread(this).start();
    }

    public long getTimeout() {
        return timeout;
    }

    public void setTimeout(long timeout) {
        this.timeout = timeout;
    }

    public WorkingMemory getWorkingMemory() {
        return workingMemory;
    }

    public void setWorkingMemory(WorkingMemory workingMemory) {
        this.workingMemory = workingMemory;
    }

    public void initWorkingMemory(WorkingMemory workingMemory) {
        setWorkingMemory(workingMemory);
    }

    /**
     * Execute the rules after the timeout
     */
    public void run() {
        Logger logger = Logger.getLogger(this.getClass().getName());
        while (isStarted()) {
            try {
                Thread.sleep(timeout);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

            if (workingMemory != null) {
                try {
                    logger.info("Execute the rules...");
                    workingMemory.fireAllRules();
                } catch (FactException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    public boolean isStarted() {
        return started;
    }

    public void setStarted(boolean started) {
        this.started = started;
    }

    /**
     * Add a file to the WorkingMemory
     * 
     * @param bytes
     *            the data of the file
     * @param fileName
     *            the name of the file
     */
    public void addFileToWorking(byte[] bytes, String fileName) {
        try {
            // Save data in $JASMINE_RULES/fileName
            saveData(bytes, fileName);

            // Configuration of the packageBuilder
            logger
                    .info("Copnfiguration of the packageBuiler for the new ruleBase...");
            PackageBuilderConfiguration pkgBuilderCfg = new PackageBuilderConfiguration();
            pkgBuilderCfg.setCompiler(PackageBuilderConfiguration.JANINO);

            PackageBuilder packageBuilder = new PackageBuilder(pkgBuilderCfg);

            RuleBase ruleBase = RuleBaseFactory.newRuleBase();

            // Load files in the JASMINE_RULES directory
            logger.info("Load files in the JASMINE_RULES directory...");
            String location = System.getenv("JASMINE_RULES");
            if (location == null) {
                throw new Exception(
                        "The environment variable JASMINE_RULES does not exist");
            }
            File directory = new File(location);

            if (!directory.exists() || !directory.isDirectory()) {
                throw new FileNotFoundException();
            }

            for (String file : directory.list()) {
                // XML files
                if (file.toLowerCase().endsWith(".xml")) {
                    packageBuilder.addPackageFromXml(new FileReader(location
                            + File.separator + file));
                    ruleBase.addPackage(packageBuilder.getPackage());
                }

                // DRL files
                if (file.toLowerCase().endsWith(".drl")) {
                    packageBuilder.addPackageFromDrl(new FileReader(location
                            + File.separator + file));
                    ruleBase.addPackage(packageBuilder.getPackage());
                }
            }

            // Recover the workingMemory and the objects in the workingMemory
            DroolsWorkingMemory droolsWorkingMemory = DroolsWorkingMemory
                    .getInstance();
            WorkingMemory workingMemory = droolsWorkingMemory
                    .getWorkingMemory();

            logger.info("Save the objects in the workingMemory...");
            List objects = workingMemory.getObjects();

            // Stop the workingMemory
            droolsWorkingMemory.setStarted(false);

            // New workingMemory
            WorkingMemory newWorkingMemory = ruleBase.newWorkingMemory();

            // Insert objects in the new workingMemory
            for (Object object : objects) {
                newWorkingMemory.assertObject(object);
            }

            // Start the new workingMemory
            logger.info("WorkingMemory is started...");
            droolsWorkingMemory.initWorkingMemory(newWorkingMemory);
            droolsWorkingMemory.setStarted(true);
            droolsWorkingMemory.run();

        } catch (IOException e) {
            e.printStackTrace();
        } catch (DroolsParserException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Save data in JASMINE_RULES/fileName
     * 
     * @param bytes
     *            the data
     * @param fileName
     *            the name of the file
     * @throws Exception
     */
    private void saveData(byte[] bytes, String fileName) throws Exception {

        logger.info("Save data in JASMINE_RULES/" + fileName);

        String location = System.getenv("JASMINE_RULES");
        if (location == null) {
            throw new Exception(
                    "The environment variable JASMINE_RULES does not exist");
        }

        File file = new File(location + File.separator + fileName);
        if (!file.exists()) {
            file.createNewFile();
        }

        FileOutputStream os = new FileOutputStream(file);

        os.write(bytes);
        os.close();
    }
}
